==> xft-sdk-1.0.0.20230518-sources.jar: com/fshows/xft/sdk/util/SignUtil.java <==
/**
 * fshows.com
 * Copyright (C) 2013-2022 All Rights Reserved.
 */
package com.fshows.xft.sdk.util;

import cn.hutool.crypto.SecureUtil;
import com.fshows.xft.sdk.exception.XftPayException;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.reflect.FieldUtils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.lang.reflect.Field;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.*;

/**
 * @author zhaoxumin
 * @version SignUtil.java, v 0.1 2023-08-25 下午11:14 zhaoxumin
 */
public class SignUtil {

    private static final String SIGN_ALGORITHMS = "SHA1WithRSA";

    private static PrivateKey getPrivateKeyFromPKCS8(String algorithm, InputStream ins) throws Exception {
        if (ins != null && !StringUtils.isEmpty(algorithm)) {
            KeyFactory keyFactory = KeyFactory.getInstance(algorithm);
            byte[] encodedKey = StreamUtil.readText(ins).getBytes();
            encodedKey = Base64.getDecoder().decode(encodedKey);
            return keyFactory.generatePrivate(new PKCS8EncodedKeySpec(encodedKey));
        } else {
            return null;
        }
    }

    private static String getContent(Map<String, String> param) {
        StringBuilder content = new StringBuilder();
        List<String> keys = new ArrayList<>(param.keySet());
        Collections.sort(keys);
        int index = 0;
        for (String key : keys) {
            String value = param.get(key);
            if (StringUtils.isNotBlank(key) && StringUtils.isNotBlank(value)) {
                content.append(index == 0 ? "" : "&").append(key).append("=").append(value);
                index++;
            }
        }
        return content.toString();
    }

    private static String getVerifiedContent(Map<String, Object> params) {
        if (params == null) {
            return null;
        }
        params.remove("sign");
        StringBuilder content = new StringBuilder();
        List<String> keys = new ArrayList<>(params.keySet());
        Collections.sort(keys);

        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key) == null ? "" : params.get(key).toString();
            content.append(i == 0 ? "" : "&").append(key).append("=").append(value);
        }

        return content.toString();
    }

    public static PublicKey getPublicKeyFromX509(String algorithm,
                                                  InputStream ins) throws Exception {
        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);

        StringWriter writer = new StringWriter();
        StreamUtil.io(new InputStreamReader(ins), writer);

        byte[] encodedKey = writer.toString().getBytes();

        encodedKey = Base64.getDecoder().decode(encodedKey);

        return keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
    }

    public static String sign(Map<String, String> data, String signType, String privateKey) {
        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(signType, new ByteArrayInputStream(privateKey.getBytes()));
            Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
            signature.initSign(priKey);
            String content = getContent(data);
            signature.update(content.getBytes(StringPool.UTF_8));
            byte[] signed = signature.sign();
            return new String(Base64.getEncoder().encode(signed));
        } catch (Exception ex) {
            return null;
        }
    }

    public static String sign(String content, String signType, String privateKey) {
        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(signType, new ByteArrayInputStream(privateKey.getBytes()));
            Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
            signature.initSign(priKey);
            signature.update(content.getBytes(StringPool.UTF_8));
            byte[] signed = signature.sign();
            return new String(Base64.getEncoder().encode(signed));
        } catch (Exception ex) {
            return null;
        }
    }

    public static Boolean verifySign(Map<String, Object> data, String signType, String sign, String publicKey) {
        try {
            String content = getVerifiedContent(data);
            PublicKey pubKey = getPublicKeyFromX509(signType,
                    new ByteArrayInputStream(publicKey.getBytes()));

            Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
            signature.initVerify(pubKey);
            signature.update(content.getBytes(StringPool.UTF_8));
            return signature.verify(Base64.getDecoder().decode(sign.getBytes()));
        } catch (Exception ex) {
            return null;
        }
    }

    public static <T> TreeMap<String, String> getParameterMap(T t) throws XftPayException {
        List<Field> fieldList = FieldUtils.getAllFieldsList(t.getClass());

        TreeMap<String, String> paramMap = new TreeMap<>();
        for (Field field : fieldList) {
            field.setAccessible(true);
            try {
                String o = (String)field.get(t);
                if (o == null) {
                    continue;
                }
                paramMap.put(field.getName(), o);
            } catch (IllegalAccessException e) {
                throw new XftPayException("");
            }
        }
        return paramMap;
    }

    /**
     * 获取签名转大写
     *
     * @param params 请求参数
     * @return sign
     */
    public static String generateSign(TreeMap<String, String> params) {
        // 遍历参数完成拼接
        StringBuilder sb = new StringBuilder(200);
        params.forEach((k, v) -> {
            if (Objects.nonNull(v)) {
                sb.append(v);
            }
        });

        // MD5取摘要后转换为大写
        return SecureUtil.md5(sb.toString()).toUpperCase();
    }
}

==> xft-sdk-1.0.0.20230914-20230830.024717-3-sources.jar: com/fshows/xft/sdk/util/SignUtil.java <==
/**
 * fshows.com
 * Copyright (C) 2013-2022 All Rights Reserved.
 */
package com.fshows.xft.sdk.util;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import com.alibaba.fastjson.JSONObject;
import com.fshows.xft.sdk.exception.XftPayException;
import com.google.common.collect.Maps;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.reflect.FieldUtils;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.lang.reflect.Field;
import java.security.KeyFactory;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author zhaoxumin
 * @version SignUtil.java, v 0.1 2023-08-25 下午11:14 zhaoxumin
 */
public class SignUtil {

    private static final String SIGN_ALGORITHMS = "SHA1WithRSA";

    private static PrivateKey getPrivateKeyFromPKCS8(String algorithm, InputStream ins) throws Exception {
        if (ins != null && !StringUtils.isEmpty(algorithm)) {
            KeyFactory keyFactory = KeyFactory.getInstance(algorithm);
            byte[] encodedKey = StreamUtil.readText(ins).getBytes();
            encodedKey = Base64.getDecoder().decode(encodedKey);
            return keyFactory.generatePrivate(new PKCS8EncodedKeySpec(encodedKey));
        } else {
            return null;
        }
    }

    private static String getContent(Map<String, String> param) {
        StringBuilder content = new StringBuilder();
        List<String> keys = new ArrayList<>(param.keySet());
        Collections.sort(keys);
        int index = 0;
        for (String key : keys) {
            String value = param.get(key);
            if (StringUtils.isNotBlank(key) && StringUtils.isNotBlank(value)) {
                content.append(index == 0 ? "" : "&").append(key).append("=").append(value);
                index++;
            }
        }
        return content.toString();
    }

    private static String getVerifiedContent(Map<String, Object> params) {
        if (params == null) {
            return null;
        }
        params.remove("sign");
        StringBuilder content = new StringBuilder();
        List<String> keys = new ArrayList<>(params.keySet());
        Collections.sort(keys);

        for (int i = 0; i < keys.size(); i++) {
            String key = keys.get(i);
            String value = params.get(key) == null ? "" : params.get(key).toString();
            content.append(i == 0 ? "" : "&").append(key).append("=").append(value);
        }

        return content.toString();
    }

    public static PublicKey getPublicKeyFromX509(String algorithm,
                                                  InputStream ins) throws Exception {
        KeyFactory keyFactory = KeyFactory.getInstance(algorithm);

        StringWriter writer = new StringWriter();
        StreamUtil.io(new InputStreamReader(ins), writer);

        byte[] encodedKey = writer.toString().getBytes();

        encodedKey = Base64.getDecoder().decode(encodedKey);

        return keyFactory.generatePublic(new X509EncodedKeySpec(encodedKey));
    }

    public static String sign(Map<String, String> data, String signType, String privateKey) {
        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(signType, new ByteArrayInputStream(privateKey.getBytes()));
            Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
            signature.initSign(priKey);
            String content = getContent(data);
            signature.update(content.getBytes(StringPool.UTF_8));
            byte[] signed = signature.sign();
            return new String(Base64.getEncoder().encode(signed));
        } catch (Exception ex) {
            return null;
        }
    }

    public static String sign(String content, String signType, String privateKey) {
        try {
            PrivateKey priKey = getPrivateKeyFromPKCS8(signType, new ByteArrayInputStream(privateKey.getBytes()));
            Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
            signature.initSign(priKey);
            signature.update(content.getBytes(StringPool.UTF_8));
            byte[] signed = signature.sign();
            return new String(Base64.getEncoder().encode(signed));
        } catch (Exception ex) {
            return null;
        }
    }

    public static Boolean verifySign(Map<String, Object> data, String signType, String sign, String publicKey) {
        try {
            String content = getVerifiedContent(data);
            PublicKey pubKey = getPublicKeyFromX509(signType,
                    new ByteArrayInputStream(publicKey.getBytes()));

            Signature signature = Signature.getInstance(SIGN_ALGORITHMS);
            signature.initVerify(pubKey);
            signature.update(content.getBytes(StringPool.UTF_8));
            return signature.verify(Base64.getDecoder().decode(sign.getBytes()));
        } catch (Exception ex) {
            return null;
        }
    }

    public static <T> TreeMap<String, String> getParameterMap(T t) throws XftPayException {
        List<Field> fieldList = FieldUtils.getAllFieldsList(t.getClass());

        TreeMap<String, String> paramMap = new TreeMap<>();
        for (Field field : fieldList) {
            field.setAccessible(true);
            try {
                Object o = field.get(t);
                if (o == null) {
                    continue;
                }

                String fieldName = field.getName();
                if ("serialVersionUID".equalsIgnoreCase(fieldName)){
                    continue;
                }
                paramMap.put(StrUtil.toUnderlineCase(fieldName), o.toString());
            } catch (IllegalAccessException e) {
                throw new XftPayException("");
            }
        }
        return paramMap;
    }

    public static TreeMap<String, String> getParameterMap(JSONObject jsonObject) {
        Map<String, Object> map = jsonObject.getInnerMap();
        TreeMap<String, String> paramMap = new TreeMap<>();
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            paramMap.put(entry.getKey(), (String)entry.getValue());
        }
        return paramMap;
    }

    /**
     * 获取签名转大写
     *
     * @param params 请求参数
     * @return sign
     */
    public static String generateSign(TreeMap<String, String> params, String key) {
        List<String> paramKeyBySort = params.keySet().stream().map(String::toLowerCase).sorted().collect(Collectors.toList());
        // 转成小写
        Map<String, String> resultMap = mapKey2LowerCase(params);
        StringBuilder paramStr = new StringBuilder();
        // 拼接字符串参数
        for (String paramKey : paramKeyBySort) {
            paramStr.append(paramKey).append("=").append(resultMap.get(paramKey)).append("&");
        }
        paramStr.append("key=").append(key);
        return SecureUtil.md5(paramStr.toString()).toUpperCase();
    }

    /**
     * key 转小写
     *
     * @param formParam
     * @return
     */
    private static Map<String, String> mapKey2LowerCase(Map<String, String> formParam) {
        Map<String, String> resultMap = Maps.newHashMap();
        for (String key : formParam.keySet()) {
            resultMap.put(key.toLowerCase(), formParam.get(key));
        }
        return resultMap;
    }
}
